"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const log_level_enum_1 = require("./log-level.enum");
class Logger {
    constructor(service, console, logLevel) {
        this.service = service;
        this.console = console;
        this.serviceName = Logger.toTag(service.constructor.name);
        this.logLevel = (undefined === logLevel) ? log_level_enum_1.LogLevel.basic : logLevel;
        this.callsCount = 0;
    }
    static toTag(str) {
        return str
            .replace(/([A-Z])([A-Z])/g, (a, pre, post) => `${pre}-${post}`)
            .replace(/([a-z])([A-Z])/g, (a, pre, post) => `${pre}-${post}`)
            .toUpperCase();
    }
    callServiceFunction(functionName, input) {
        return __awaiter(this, void 0, void 0, function* () {
            const callNumber = this.callsCount++;
            const tag = Logger.toTag(functionName);
            let response;
            this.log(tag, callNumber, `[CALLED] ${this.parseFuncInput(input)}`, log_level_enum_1.LogLevel.basic);
            try {
                response = yield this.service[functionName](input);
            }
            catch (err) {
                this.log(tag, callNumber, `[ERROR] ${err.stack}`, log_level_enum_1.LogLevel.error);
                throw err;
            }
            this.log(tag, callNumber, `[OK] ${this.parseFuncOutput(response)}`, log_level_enum_1.LogLevel.basic);
            return response;
        });
    }
    log(tag, callNumber, message, level) {
        if (level > this.logLevel) {
            return;
        }
        const parsedMessage = typeof message === 'string' ? message : JSON.stringify(message);
        this.console.log(`[SERV:${this.serviceName}] [FUNC:${tag}] [CALL:${callNumber}] ${parsedMessage}`);
    }
}
exports.default = Logger;
